/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */
 
#ifndef _SDOM_NAMED_NODE_MAP_H
#define _SDOM_NAMED_NODE_MAP_H  

#include "../Internal2/STree/myUtilsRoot.h"
#include "SDOM_Node.h"


class SDOM_Node;
class SucTree;
class SDOM_Attribute;

/**
 *  <code>SDOM_NamedNodeMap</code>s  are used to
 * represent collections of nodes that can be accessed by name.
 *
 * Note that <code>SDOM_NamedNodeMap</code> does not inherit from <code>MyDOMNodeList</code>;
 * <code>SDOM_NamedNodeMap</code>s are not maintained in any particular order.
 * Nodes contained in a <code>SDOM_NamedNodeMap</code> may
 * also be accessed by an ordinal index, but this is simply to allow
 * convenient enumeration of the contents, and
 * does not imply that the DOM specifies an order to these Nodes.
 *
 * @since DOM Level 1
 */
class SDOM_NamedNodeMap{
    
static SDOM_NamedNodeMap *NNB;

public:

/**
* Default Constructor
*/
 SDOM_NamedNodeMap():attCount(0){}
 
 /**
 * Main Constructor
 */
 SDOM_NamedNodeMap(SucTree * sucTree, nodeValue, int);
 
 /**
 * Copy Constructor
 */
  SDOM_NamedNodeMap(const SDOM_NamedNodeMap & nMap);
 
 /**
 * destructor
 */
 ~SDOM_NamedNodeMap(){}
 
 static SDOM_NamedNodeMap * instance(SucTree *n,nodeValue,int);
    
 /**
     * Adds a node using its <code>nodeName</code> attribute.
     *
     * <br>As the <code>nodeName</code> attribute is used to derive the name
     * which the node must be stored under, multiple nodes of certain types
     * (those that have a "special" string value) cannot be stored as the names
     * would clash. This is seen as preferable to allowing nodes to be aliased.
     * @param arg A node to store in a named node map. The node will later be
     *   accessible using the value of the <code>nodeName</code> attribute of
     *   the node. If a node with that name is already present in the map, it
     *   is replaced by the new one.
     * @return If the new <code>SDOM_Node</code> replaces an existing node the
     *   replaced <code>SDOM_Node</code> is returned,
     *   otherwise <code>null</code> is returned.
     * @since DOM Level 1
  */
 SDOM_Node   *setNamedItem(SDOM_Node *arg);

 /**
     * Returns the <code>index</code> of the item in the map.
     *
     * If <code>index</code>
     * is greater than or equal to the number of nodes in the map, this returns
     * <code>null</code>.
     * @param index Index into the map.
     * @return The node at the <code>index</code>th position in the
     *   <code>SDOM_NamedNodeMap</code>, or <code>null</code> if that is not a valid
     *   index.
     * @since DOM Level 1
     */
 SDOM_Node     *item(int index) const;
 
 /**
     * Retrieves a node specified by name.
     *
     * @param name The <code>nodeName</code> of a node to retrieve.
     * @return A <code>SDOM_Node</code> (of any type) with the specified <code>nodeName</code>, or
     *   <code>null</code> if it does not identify any node in
     *   the map.
     * @since DOM Level 1
     */
 SDOM_Node   *getNamedItem(const char *name) const;

 /**
     * The number of nodes in the map.
     *
     * The range of valid child node indices is
     * 0 to <code>length-1</code> inclusive.
     * @since DOM Level 1
   */
 inline virtual int getLength() const{
    return attCount;
 }

 /**
  * Dynamic methods not supported in SDOM.
  */
    SDOM_Node    *removeNamedItem(const char *name);
    
    SDOM_Node   *getNamedItemNS(const char *namespaceURI,
	                                       const char *localName) const;
    /**
  * Dynamic methods not supported in SDOM.
  */
    SDOM_Node   *setNamedItemNS(SDOM_Node *arg);
    
    /**
  * Dynamic methods not supported in SDOM.
  */
    SDOM_Node     *removeNamedItemNS(const char *namespaceURI,
					    const char *localName);
  /**
  * Extension method to set parent node of SDOM_NamedNodeMap
  * @param node ID. If node is not an element parent node should not be updated.
  * Need to test this method.
  */
  void setParentNode(nodeValue);
  void setParentNode(SDOM_Node*);
private:

  nodeValue parentNode;
  SucTree * sucTree;
  SDOM_Attribute* attNode; 
    SDOM_Attribute* attrSDOM_Node;
  
  int otherNode;
  int attPosition; 
  int attCount;
  char* dummy;

};

class  AttrNamedNodeMap: public SDOM_NamedNodeMap{
public:

  AttrNamedNodeMap();
  AttrNamedNodeMap(SDOM_Node * node);
  ~AttrNamedNodeMap();

 private:
};


#endif
