/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */
 
#ifndef _SDOM_NODE_H
#define _SDOM_NODE_H
#include "Type.h"
#include "SDOM_NamedNodeMap.h"
#include "SDOM_Exception.h"
#include "SDOM_NodeList.h"
#include "../Internal2/STree/myUtilsRoot.h"
#include "../Internal2/SucTree.h"
#include <new> // for size_t
#include <vector>
#include <exception>
#include <cstdlib>

class SDOM_NamedNodeMap;
class SDOM_NodeList;
class SDOM_Element;
class SDOM_Document;
class SDOM_Text;
class SDOM_Node;
class SDOM_CDataSection;
class SDOM_PI;
class SDOM_Attribute;
class SDOM_EntityRef ;
class SDOM_DocType;
class CharacterData;
//class SDOM_Exception;

/**
 * The <code>DOMNode</code> we rename in SDOM as <code>SDOM_Node</code> interface,
 * it is the primary datatype for the entire Document Object Model. 
 * It represents a single node in the document tree.
 * While all objects implementing the <code>DOMNode</code> interface expose
 * methods for dealing with children, not all objects implementing the
 * <code>DOMNode</code> interface may have children. For example,
 * <code>DOMText</code> nodes may not have children.
 * <p>The attributes <code>nodeName</code>, <code>nodeValue</code> and
 * <code>attributes</code> are included as a mechanism to get at node
 * information without casting down to the specific derived interface. In
 * cases where there is no obvious mapping of these attributes for a
 * specific <code>nodeType</code> (e.g., <code>nodeValue</code> for an
 * <code>DOMElement</code> or <code>attributes</code> for a <code>DOMComment</code>
 * ), this returns <code>null</code>. Note that the specialized interfaces
 * may contain additional and more convenient mechanisms to get and set the
 * relevant information.
 * <p>See also the <a href='http://www.w3.org/TR/2000/REC-DOM-Level-2-Core-20001113'>Document Object Model (DOM) Level 2 Core Specification</a>.
 *
 * @since DOM Level 1
 */
class SDOM_Node{
 static SDOM_NamedNodeMap *NullNamedNodeMap;
static SDOM_Node * firstNode;
 public:
   static SDOM_NodeList *NullDOMNodeList;
   
   /**SucTree *myTree, nodeValue n
    * Default Constructor
   */
  SDOM_Node(){}
   
  /**
  * Main Constructor
  * @param feature the internal node ID
  */
 // SDOM_Node(MyNodeImpl* node);
  
  SDOM_Node(SucTree *myTree, nodeValue n)  ;
  /**
  * Copy Constructor
  * @param feature the internal node ID
  */
  SDOM_Node(const SDOM_Node & nn);
    
  /**
  * Destructor
  */
  ~SDOM_Node();
    
    
  /**
  * Extension method from SDOM to wrap internal node to DOM node interface
  * @param feature the internal node ID
  */
  //  SDOM_Node*           wrap(MyNodeImpl* node);
    
    /**
     * The name of this node, depending on its type; see the table above.
     * @since DOM Level 1
     */
    virtual const char *        getNodeName() const ;
    
    /**
     * The value of this node, depending on its type.
     * @since DOM Level 1
     */
    virtual const char *        getNodeValue() const;
    
    /**
     * An short value representing the type of the underlying object.
     * @since DOM Level 1
     */
    virtual short               getNodeType() const ;
    
    /**
     * Returns the parent of this node as a new object.
     *
     * All nodes, except <code>DOMDocument</code>, <code>Entity</code>
     *and <code>DOMAttr</code> may have a parent.
     * @since DOM Level 1
     */
    virtual SDOM_Node    *getParentNode() const ;
    
    /**
     * Gets a <code>SDOM_NodeList</code> that contains all children of this node.
     *
     * If there
     * are no children, this is a <code>SDOM_NodeList</code> containing no nodes.
     * We differ from the DOM spec in that it is a static snapshot of the content of the node. 
     * This is true for every <code>DOMNodeList</code>,
     * including the ones returned by the <code>getElementsByTagName</code>
     * method.
     * @since DOM Level 1
     */
    virtual SDOM_NodeList       *getChildNodes() const ;

    /**
     * Returns the first child of this node.
     *
     * If there is no such node, this returns <code>null</code>.
     * @since DOM Level 1
     */
    virtual SDOM_Node    *getFirstChild() const;

     /**
     * Returns the last child of this node.
     *
     * If there is no such node, this returns <code>null</code>.
     * @since DOM Level 1
     */
    virtual SDOM_Node    *getLastChild() const;
    
     /**
     * Returns the node immediately preceding this node.
     *
     * If there is no such node, this returns <code>null</code>.
     * @since DOM Level 1
     */
    virtual SDOM_Node    *getPreviousSibling() const;
    
    /**
     * Returns the node immediately following this node.
     *
     * If there is no such node, this returns <code>null</code>.
     * @since DOM Level 1
     */
    virtual SDOM_Node    *getNextSibling() const;
    
    /**
     * Returns a <code>SDOM_NamedNodeMap</code> containing the attributes of this node (if it
     * is an <code>DOMElement</code>) or <code>null</code> otherwise.
     * @since DOM Level 1
     */
    virtual SDOM_NamedNodeMap        *getAttributes() const;

    /**
     * Returns the <code>SDOM_Document</code> object associated with this node.
     * @since DOM Level 1
     */
    virtual SDOM_Document *getOwnerDocument() const;

    /**
     * Returns Null. We do not support this method.
     */
    virtual SDOM_Node    *cloneNode(bool deep) const;

    /**
     * Returns Null. We do not support this method.
     */
     virtual SDOM_Node    *insertBefore(SDOM_Node *newChild,
                                          SDOM_Node *refChild) ;

     /**
     * Returns Null. We do not support this method.
     */
     virtual SDOM_Node    *replaceChild(SDOM_Node *newChild,
                                     SDOM_Node *oldChild) ;

    /**
     * Returns Null. We do not support this method.
     */
    virtual SDOM_Node    *removeChild(SDOM_Node *oldChild) ;

    /**
     * Returns Null. We do not support this method.
     */
    virtual SDOM_Node    *appendChild(SDOM_Node *newChild) ;

     /**
     *  Returns whether a node has any children.
     *
     * @return  <code>true</code> if the node has any children,
     *   <code>false</code> if the node has no children.
     * @since DOM Level 1
     */
    virtual bool                hasChildNodes() const ;
    virtual bool 		hasNextSibling() const ;

    /**
     * Method not supported.
     */
    virtual void                setNodeValue(const char  *nodeValuei);

    /**
     * Method not supported.
     */
    virtual void normalize();

    /**
     * Tests whether the DOM implementation implements a specific
     * feature and that feature is supported by this node.
     *
     * @param feature The string of the feature to test.
     * @param version This is the version number of the feature to test. In
     * Level 2, version 1, this is the string "2.0". If the version is not
     * specified, supporting any version of the feature will cause the
     * method to return <code>true</code>.
     * @return Returns <code>true</code> if the specified feature is supported
     * on this node, <code>false</code> otherwise.
     * @since DOM Level 2
    */
    virtual bool                isSupported(const char *feature,
	                                       const char *version) const ;

    /**
     * Returns char array of the <em>namespace URI</em> of
     * this node, or <code>null</code> if it is unspecified.
     * For nodes of any type other than <CODE>ELEMENT_NODE</CODE> and
     * <CODE>ATTRIBUTE_NODE</CODE> this is always <CODE>null</CODE>.
     * @since DOM Level 2
    */
    virtual const char *        getNamespaceURI() const ;

    /**
     * Returns char array of the <em>namespace prefix</em>
     * of this node, or <code>null</code> if it is unspecified.
     *
     * @since DOM Level 2
     */
    virtual const char *        getPrefix() const ;

    /**
     * Returns the local part of the <em>qualified name</em> of this node.
     * @since DOM Level 2
     */
    virtual const char *        getLocalName() const ;

    /**
     * Method not supported.
    */
    virtual void                setPrefix(const char * prefix) ;

    /**
     *  Returns whether this node (if it is an element) has any attributes.
     * @return <code>true</code> if this node has any attributes,
     *   <code>false</code> otherwise.
     * @since DOM Level 2
    */
    virtual bool                hasAttributes() const ;

        /**
     * Returns whether this node is the same node as the given one.
     * <br>This method provides a way to determine whether two
     * <code>Nodes</code> are referring to the same Node, with same ID.
     *
     * @param other The node to test against.
     * @return Returns <code>true</code> if the nodes are the same,
     *   <code>false</code> otherwise.
     * @since DOM Level 3
     */
    virtual bool                isSameNode(SDOM_Node* other) const ;

     /**
     * Tests whether two nodes are equal.
     * <br>This method tests for equality of nodes, not sameness (i.e.,
     * whether the two nodes are pointers to the same object) which can be
     * tested with <code>DOMNode::isSameNode</code>. All nodes that are the same
     * will also be equal, though the reverse may not be true.
     * <br>Two nodes are equal if and only if the following conditions are
     * satisfied: The two nodes are of the same type.The following string
     * attributes are equal: <code>nodeName</code>, <code>localName</code>,
     * <code>namespaceURI</code>, <code>prefix</code>, <code>nodeValue</code>
     * , <code>baseURI</code>. This is: they are both <code>null</code>, or
     * they have the same length and are character for character identical.
     * The <code>attributes</code> <code>SDOM_NamedNodeMaps</code> are equal.
     * This is: they are both <code>null</code>, or they have the same
     * length and for each node that exists in one map there is a node that
     * exists in the other map and is equal, although not necessarily at the
     * same index.The <code>childNodes</code> <code>MyNodeLists</code> are
     * equal. This is: they are both <code>null</code>, or they have the
     * same length and contain equal nodes at the same index. This is true
     * for <code>DOMAttr</code> nodes as for any other type of node. 
     * <br>For two <code>DocType</code> nodes to be equal, the following
     * conditions must also be satisfied: The following string attributes
     * are equal: <code>publicId</code>, <code>systemId</code>,
     * <code>internalSubset</code>.The <code>entities</code>
     * <code>SDOM_NamedNodeMaps</code> are equal.The <code>notations</code>
     * <code>SDOM_NamedNodeMaps</code> are equal.
     * <br>On the other hand, the following do not affect equality: the
     * <code>ownerDocument</code> attribute, the <code>specified</code>
     * attribute for <code>DOMAttr</code> nodes, the
     * <code>isWhitespaceInElementContent</code> attribute for
     * <code>DOMText</code> nodes.
     *
     * @param arg The node to compare equality with.
     * @return If the nodes, and possibly subtrees are equal,
     *   <code>true</code> otherwise <code>false</code>.
     * @since DOM Level 3
     */
    virtual bool                isEqualNode(SDOM_Node* arg) const;

    /**
     * Method not supported.
    */
    virtual void             setUserData(const char* key,
                                          void* data,
                                          void* handler){}

    /**
     * Method not supported.
    */
    virtual void*               getUserData(const char* key) const ;

     /**
     * This not supported yet.
     * Howver the absolute base URI of this node or <code>null</code> if undefined.
     * This value is computed according to . However, when the
     * <code>DOMDocument</code> supports the feature "HTML" , the base URI is
     * computed using first the value of the href attribute of the HTML BASE
     * element if any, and the value of the <code>documentURI</code>
     * attribute from the <code>DOMDocument</code> interface otherwise.
     *
     * <br> When the node is an <code>DOMElement</code>, a <code>DOMDocument</code>
     * or a a <code>DOMProcessingInstruction</code>, this attribute represents
     * the properties [base URI] defined in . When the node is a
     * <code>DOMNotation</code>, an <code>DOMEntity</code>, or an
     * <code>DOMEntityReference</code>, this attribute represents the
     * properties [declaration base URI].
     * @since DOM Level 3
     */
    virtual const char*         getBaseURI() const ;

      /**
     * Compares a node with this node with regard to their position in the
     * tree and according to the document order.
     *
     * @param other The node to compare against this node.
     * @return Returns how the given node is positioned relatively to this
     *   node.
     * @since DOM Level 3
     */
    virtual short               compareTreePosition(SDOM_Node* other) const ;

     /**
     * NOTE Status - CDataSection text not included in the returned string.
     * This attribute returns the text content of this node and its
     * descendants. When it is defined to be null, setting it has no effect.
     * When set, any possible children this node may have are removed and
     * replaced by a single <code>DOMText</code> node containing the string
     * this attribute is set to. On getting, no serialization is performed,
     * the returned string does not contain any markup. No whitespace
     * normalization is performed, the returned string does not contain the
     * element content whitespaces . Similarly, on setting, no parsing is
     * performed either, the input string is taken as pure textual content.
     *
     * <br>The string returned is made of the text content of this node
     * depending on its type, as defined below:
     * <table border='1'>
     * <tr>
     * <th>Node type</th>
     * <th>Content</th>
     * </tr>
     * <tr>
     * <td valign='top' rowspan='1' colspan='1'>
     * ELEMENT_NODE, ENTITY_NODE, ENTITY_REFERENCE_NODE,
     * DOCUMENT_FRAGMENT_NODE</td>
     * <td valign='top' rowspan='1' colspan='1'>concatenation of the <code>textContent</code>
     * attribute value of every child node, excluding COMMENT_NODE and
     * PROCESSING_INSTRUCTION_NODE nodes</td>
     * </tr>
     * <tr>
     * <td valign='top' rowspan='1' colspan='1'>ATTRIBUTE_NODE, TEXT_NODE,
     * CDATA_SECTION_NODE, COMMENT_NODE, PROCESSING_INSTRUCTION_NODE</td>
     * <td valign='top' rowspan='1' colspan='1'>
     * <code>nodeValue</code></td>
     * </tr>
     * <tr>
     * <td valign='top' rowspan='1' colspan='1'>DOCUMENT_NODE, DOCUMENT_TYPE_NODE, NOTATION_NODE</td>
     * <td valign='top' rowspan='1' colspan='1'>
     * null</td>
     * </tr>
     * </table>
     * @since DOM Level 3
     */
    virtual const char*         getTextContent() const ;

    /**
     * Method not supported.
    */
    virtual void                setTextContent(const char* textContent) ;

     /**
     * NOTE Status - Not done
     * Look up the prefix associated to the given namespace URI, starting from
     * this node.
     *
     * @since DOM Level 3
     */
    virtual const char*         lookupNamespacePrefix(const char* namespaceURI,
                                                         bool useDefault) const ;

     /**
     * NOTE Status - Not done
     * This method checks if the specified <code>namespaceURI</code> is the
     * default namespace or not.
     *
     * @param namespaceURI The namespace URI to look for.
     * @return  <code>true</code> if the specified <code>namespaceURI</code>
     *   is the default namespace, <code>false</code> otherwise.
     * @since DOM Level 3
     */
    virtual bool                isDefaultNamespace(const char* namespaceURI) const ;

     /**
     * NOTE Status - Not done
     * This method checks if the specified <code>namespaceURI</code> is the
     * default namespace or not.
     *
     * @param namespaceURI The namespace URI to look for.
     * @return  <code>true</code> if the specified <code>namespaceURI</code>
     *   is the default namespace, <code>false</code> otherwise.
     * @since DOM Level 3
     */
    virtual const char*         lookupNamespaceURI(const char* prefix) const  ;

     /**
     * This method makes available a <code>SDOM_Node</code>'s specialized interface
     *
     * @param feature The name of the feature requested (case-insensitive).
     * @return Returns an alternate <code>SDOM_Node</code> which implements the
     *   specialized APIs of the specified feature, if any, or
     *   <code>null</code> if there is no alternate <code>SDOM_Node</code> which
     *   implements interfaces associated with that feature. Any alternate
     *   <code>SDOM_Node</code> returned by this method must delegate to the
     *   primary core <code>SDOM_Node</code> and not return results inconsistent
     *   with the primary core <code>SDOM_Node</code> such as <code>key</code>,
     *   <code>attributes</code>, <code>childNodes</code>, etc.
     * @since DOM Level 3
     */
    virtual SDOM_Node*   getInterface(const char* feature) ;
    
    /**
    * Extension methods - Deletes the node
    */
    virtual void                release();
    
    /**
    * Extension methods - returns pre-order node number (ID)
    */
  nodeValue getNodeNr();
  void setNodeNr(nodeValue nodei){
	nodeNr = nodei;
  }
 
  void setSucTree(SucTree * sTree){
    sucTree = sTree;
  }
  
  unsigned int hashCode() const{
  int num = sucTree->getStree().getOtherNodeNumber(nodeNr.node_n);
  if(num!=-1){
    short scode=sucTree->getNameCodeDS().getShortCode(num);
    if(scode<13)
      return scode;//node has no real hashcode 
    return sucTree->getNamepool().decode(scode);
  }
  return 0;
}

    virtual const char* getStringValue() const{
      
      int val; int otherNode=0;
      switch(getNodeType()){
      case TEXT: 
	val= sucTree->getStree().getTextNodeNumber(nodeNr.node_n);
	return sucTree->getTextDS().getTextString(val);
      case DOCUMENT: return "";
      case ELEMENT: return "";
      case ATTRIBUTE: otherNode= sucTree->getStree().getOtherNodeNumber(nodeNr.node_n);
	return "";	
      case PROCESSING_INSTRUCTION: return "";
      case DOCUMENT_TYPE: return "";
      case ENTITY_REFERENCE:return "";
      case ENTITY: return "";
      case COMMENT:
      case CDATA_SECTION: otherNode= sucTree->getStree().getOtherNodeNumber(nodeNr.node_n);
	val=sucTree->getAttributeDS().getAttNode(otherNode,1);
	return sucTree->getTextDS().getString(val);;
      default:
	return "unknown";
      }
    }

  SucTree *getSucTree(){return sucTree;}
  void setNode(SDOM_Node* node);


 static void* operator new (size_t size);
 static void operator delete (void *p);

	

 // static std::vector<void*> * freeBlockVector; 
 // const static int BLOCK_SIZE = 100;


protected:
  //MyNodeImpl* myNodeImpl; 
/**< Internal representation of node - consists of doc pointer and 
  				* 2 integers (preorder number, tree-position number) 
			*/
	nodeValue nodeNr;
	SucTree * sucTree;

	SDOM_Node * previous;
	SDOM_Node * next; 

 
};




//////////////////////////////////////////////////////
/**
 * The <code>DOMDocument</code> interface rename as SDOM_Document in SDOM, represents the entire XML
 * document. Conceptually, it is the root of the document tree, and provides
 * the primary access to the document's data.
 * <p>Since elements, text nodes, comments, processing instructions, etc.
 * cannot exist outside the context of a <code>DOMDocument</code>. In SDOM the
 * the dynamic methods are not supported.
 * <p>See also the <a href='http://www.w3.org/TR/2000/REC-DOM-Level-2-Core-20001113'>Document Object Model (DOM) Level 2 Core Specification</a>.
 */
class SDOM_Document: public SDOM_Node{
public:
  /**
  * Default Constructor
  */
  SDOM_Document();
  
  /**
  * Main Constructor
  * @param feature the internal node ID
  */
  SDOM_Document(SucTree *myTree, nodeValue n);

//(MyNodeImpl* n);
  
  /**
  * Destructor
  */
  ~SDOM_Document(){
	/*need soemthing in here*/
   }
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_Element    *createElement(const char *tagName);
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_Node       *createDocumentFragment();
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_Text       *createTextNode(const char *data);
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_Node       *createComment(const char *data);
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_CDataSection      *createCDATASection(const char *data);
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_PI        *createProcessingInstruction(const char *target,const char *data);
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_Attribute      *createAttribute(const char *name);
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_EntityRef *createEntityReference(const char *name);


      SDOM_Node     *importNode(SDOM_Node *importedNode, bool deep);

     SDOM_Element         *createElementNS(const char *namespaceURI,
 	                                              const char *qualifiedName);
 
   
     SDOM_Attribute        *createAttributeNS(const char *namespaceURI,
 	                                            const char *qualifiedName);
 
   /**
     * The Document Type Declaration (see <code>DOMDocumentType</code>)
     * associated with this document. For XML
     * documents without a document type declaration this returns
     * <code>null</code>. The DOM Level 2 does not support editing the
     * Document Type Declaration. <code>docType</code> cannot be altered in
     * any way, including through the use of methods inherited from the
     * <code>DOMNode</code> interface, such as <code>insertNode</code> or
     * <code>removeNode</code>.
     * @since DOM Level 1
     */
  SDOM_DocType   *getDoctype() const;
   
  /**
   * This is a convenience attribute that allows direct access to the child
   * node that is the root element of the document.
   * @since DOM Level 1
  */
  SDOM_Node    *getDocumentElement();
  
  short               getNodeType() const{return DOCUMENT;}
  const char          *getNodeValue() const;
  bool                isDefaultNamespace(const char* namespaceURI) const {} //needs to be done
  
    /**
     * Returns a <code>SDOM_NodeList</code> of all the <code>DOMElement(s)</code> with a
     * given tag name in the order in which they are encountered in a
     * preorder traversal of the <code>DOMDocument</code> tree.
     *
     * @param tagname The name of the tag to match on. The special value "*"
     *   matches all tags.
     * @return A new <code>DOMNodeList</code> object containing all the matched
     *   <code>DOMElement(s)</code>.
     * @since DOM Level 1
     */
  SDOM_NodeList       *getElementsByTagName(const char* tagname) const;
  
     /**
     * Returns a <code>SDOM_NodeList</code> of all the <code>DOMElement(s)</code> with a
     * given local name and namespace URI in the order in which they are
     * encountered in a preorder traversal of the <code>DOMDocument</code> tree.
     * @param namespaceURI The namespace URI of the elements to match on. The
     *   special value "*" matches all namespaces.
     * @param localName The local name of the elements to match on. The
     *   special value "*" matches all local names.
     * @return A new <code>SDOM_NodeList</code> object containing all the matched
     *   <code>DOMElement(s)</code>.
     * @since DOM Level 2
     */
  SDOM_NodeList       *getElementsByTagNameNS(const char* URI,const char* tagname) const;
  
    /**
     * Returns the <code>DOMElement</code> whose <code>ID</code> is given by
     * <code>elementId</code>. If no such element exists, returns
     * <code>null</code>. Behavior is not defined if more than one element
     * has this <code>ID</code>. The DOM implementation must have
     * information that says which attributes are of type ID. Attributes
     * with the name "ID" are not of type ID unless so defined.
     * Implementations that do not know whether attributes are of type ID or
     * not are expected to return <code>null</code>.
     * @param elementId The unique <code>id</code> value for an element.
     * @return The matching element.
     * @since DOM Level 2
     */
  SDOM_Element *getElementById(const char *elementId) const;
  
      /** @name Functions introduced in DOM Level 3. */
    //@{

    /**
     * An attribute specifying the actual encoding of this document. This is
     * <code>null</code> otherwise.
     * <br> This attribute represents the property [character encoding scheme]
     * defined in.
     *
     * @since DOM Level 3
     */
  const char          *getActualEncoding() const;
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  const char          *setActualEncoding() const;
  
  /**
     * An attribute specifying, as part of the XML declaration, the encoding
     * of this document. This is <code>null</code> when unspecified.
     *
     *
     * @since DOM Level 3
  */
  const char          *getEncoding() const;
  
    /**
     * An attribute specifying, as part of the XML declaration, whether this
     * document is standalone.
     * <br> This attribute represents the property [standalone] defined in .
     *
     * @since DOM Level 3
     */
  bool                getStandalone() const;
  
    /**
     * An attribute specifying, as part of the XML declaration, the version
     * number of this document. This is <code>null</code> when unspecified.
     * <br> This attribute represents the property [version] defined in .
     *
     * @since DOM Level 3
   */
  const double        getVersion() const;
  
   /**
     * The location of the document or <code>null</code> if undefined.
     * <br>Beware that when the <code>DOMDocument</code> supports the feature
     * "HTML" , the href attribute of the HTML BASE element takes precedence
     * over this attribute.
     *
     * @since DOM Level 3
     */
  const char          *getDocumentURI() const;
  
   /**
     * An attribute specifying whether errors checking is enforced or not.
     * When set to <code>false</code>, the implementation is free to not
     * test every possible error case normally defined on DOM operations,
     * and not raise any <code>DOMException</code>. In case of error, the
     * behavior is undefined. This attribute is <code>true</code> by
     * defaults.
     *
     * @since DOM Level 3
     */
  bool                getStrictErrorChecking() const;

  SDOM_Node *	      getNodeByNumber(unsigned int nodeNumber) const;

 private:
  SDOM_DocType *myDocType;/**< Pointer to DocType node */
  SDOM_Element *rootElem;  /**< Pointer of root Element node - NOTE - Might not need this */
};



////////////////////////////////////////////////////////
/**
 * SDOM_Element Class represents the ELementOverNodeInfo node.
 * <p>Elements may have attributes associated with them; since the
 * <code>SDOM_Element</code> interface inherits from <code>SDOM_Node</code>, the generic
 *  <code>SDOM_Node</code> interface method <code>getAttributes</code> may be used
 * to retrieve the set of all attributes for an element.  There are methods on
 *  the <code>SDOM_Element</code> interface to retrieve either an <code>DOMAttr</code>
 *  object by name or an attribute value by name. In XML, where an attribute
 * value may contain entity references, an <code>SDOM_Attribute</code> object should be
 * retrieved to examine the possibly fairly complex sub-tree representing the
 * attribute value. On the other hand, in HTML, where all attributes have
 * simple string values, methods to directly access an attribute value can
 * safely be used as a convenience.
 *
 * @since DOM Level 1
 */


class SDOM_Element: public SDOM_Node{

public:
  
  /**
  * Default Constructor
  * @param feature the internal node ID
  */
  SDOM_Element();
  
  /**
  * Main Constructor
  * @param feature the internal node ID
  */
  SDOM_Element(SucTree *myTree, nodeValue n);//(MyNodeImpl* node);
   
  /**
  * Destructor
  */
  ~SDOM_Element();
  
  short getNodeType() const;
  const char * getNodeValue() const;
  
  /**
   * The name of the element. If <code>NodeNodeInfo.localName</code> is different from <code>null</code>
   *
   * For example, in: &lt;elementExample
   * id="demo"&gt;  ... &lt;/elementExample&gt; , <code>tagName</code> has
   * the value <code>"elementExample"</code>. Note that this is
   * case-preserving in XML, as are all of the operations of the DOM.
   * @since DOM Level 1
  */
  const char * getTagName() const;
  
  /**
     * Retrieves an attribute value by name.
     *
     * @param name The name of the attribute to retrieve.
     * @return The <code>SDOM_Attribute</code> value as a string, or the empty  string if
     *   that attribute does not have a specified or default value.
     * @since DOM Level 1
   */
  const char * getAttribute(const char* nm) const;
  
  /**
     * Retrieves an <code>SDOM_Attribute</code> node by name.
     *
     * @param name The name (<CODE>nodeName</CODE>) of the attribute to retrieve.
     * @return The <code>DOMAttr</code> node with the specified name (<CODE>nodeName</CODE>) or
     *   <code>null</code> if there is no such attribute.
     * @since DOM Level 1
     */
  SDOM_Node *getAttributeNode(const char* nm) const;
  
  
  /**
     * Retrieves an attribute value by local name and namespace URI.
     *
     * @param namespaceURI The <em>namespace URI</em> of
     *    the attribute to retrieve.
     * @param localName The <em>local name</em> of the
     *    attribute to retrieve.
     * @return The <code>DOMAttr</code> value as a string, or an <CODE>null</CODE> if
     *    that attribute does not have a specified or default value.
     * @since DOM Level 2
   */
  const char* getAttributeNS(const char* uri,const char* nm) const;
  
  
  /**
     * Retrieves an <code>SDOM_Attribute</code> node by local name and namespace URI.
     *
     * @param namespaceURI The <em>namespace URI</em> of
     *    the attribute to retrieve.
     * @param localName The <em>local name</em> of the
     *    attribute to retrieve.
     * @return The <code>SDOM_Attribute</code> node with the specified attribute local
     *    name and namespace URI or <code>null</code> if there is no such attribute.
     * @since DOM Level 2
  */
  SDOM_Node *getAttributeNodeNS(const char* uri,const char* nm) const;
  
  /**
     * Returns a <code>SDOM_NodeList</code> of all descendant elements with a given
     * tag name, in the order in which they would be encountered in a preorder
     * traversal of the <code>SDOM_Element</code> tree.
     *
     * @param name The name of the tag to match on. The special value "*"
     *   matches all tags.
     * @return A list of matching <code>DOMElement</code> nodes.
     * @since DOM Level 1
  */
  SDOM_NodeList *getElementsByTagName(const char* nm);
  
  
  SDOM_NodeList *getElementsByTagNameNS(const char* uri, const char* nm);
  
  
  /**
     * Returns <code>true</code> when an attribute with a given name is
     * specified on this element or has a default value, <code>false</code>
     * otherwise.
     * @param name The name of the attribute to look for.
     * @return <code>true</code> if an attribute with the given name is
     *   specified on this element or has a default value, <code>false</code>
     *    otherwise.
     * @since DOM Level 2
     */  
  bool           hasAttribute(const char* nm);
  
  /**
     * Returns <code>true</code> when an attribute with a given local name and
     * namespace URI is specified on this element or has a default value,
     * <code>false</code> otherwise. HTML-only DOM implementations do not
     * need to implement this method.
     * @param namespaceURI The namespace URI of the attribute to look for.
     * @param localName The local name of the attribute to look for.
     * @return <code>true</code> if an attribute with the given local name
     *   and namespace URI is specified or has a default value on this
     *   element, <code>false</code> otherwise.
     * @since DOM Level 2
     */
  bool           hasAttributeNS(const char* uri, const char* nm);
  
  /**
     * NOTE - Status: Not implemented yet.
     * Returns the type information associated with this element.
     *
     *
     * @return the <code>DOMTypeInfo</code> associated with this element
     * @since DOM level 3
     */
  virtual const SDOM_Node* getTypeInfo() const;
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  void             setAttribute(const char *name,const char *value){}
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_Node *setAttributeNode(SDOM_Node *newAttr);
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_Node *removeAttributeNode(SDOM_Node *oldAttr);
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  void           removeAttribute(const char *name){}
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  void           setAttributeNS(const char *namespaceURI,
                                            const char *qualifiedName, const char *value){}
  /**
  * Dynamic methods not supported in SDOM.
  */
  void           removeAttributeNS(const char *namespaceURI,
                                                const char *localName){}
  /**
  * Dynamic methods not supported in SDOM.
  */
  SDOM_Node *setAttributeNodeNS(SDOM_Node *newAttr);
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  void setIdAttribute(const char* name){}
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  void setIdAttributeNS(const char* namespaceURI, const char* localName){}
  
  /**
  * Dynamic methods not supported in SDOM.
  */
  void setIdAttributeNode(const SDOM_Node *idAttr){}
};


//////////////////////////////////////
/**
 * The <code>SDOM_Attribute</code> class refers to an attribute of an XML element.
 *
 * Typically the allowable values for the
 * attribute are defined in a documenttype definition.
 * <p><code>SDOM_Attribute</code> objects inherit the <code>DOMNode</code>  interface, but
 * since attributes are not actually child nodes of the elements they are associated with, the
 * DOM does not consider them part of the document  tree.  Thus, the
 * <code>SDOM_Node</code> attributes <code>parentNode</code>,
 * <code>previousSibling</code>, and <code>nextSibling</code> have a  null
 * value for <code>SDOM_Attribute</code> objects. The DOM takes the  view that
 * attributes are properties of elements rather than having a  separate
 * identity from the elements they are associated with;  this should make it
 * more efficient to implement such features as default attributes associated
 * with all elements of a  given type.  In short, users of the DOM
 * need to be aware that  <code>SDOM_Attribute</code> nodes have some things in  common
 * with other objects inheriting the <code>SDOM_Node</code> interface, but they
 * also are quite distinct.
 *
 * @since DOM Level 1
 */
class SDOM_Attribute: public SDOM_Node{
public:
  
  /**
  * Default Constructor
  */
  SDOM_Attribute();
  
  /**
  * Main Constructor
  * @param the internal node ID
  */
  SDOM_Attribute(SucTree *myTree, nodeValue n);//(MyNodeImpl* n);
  
  /**
  * Destructor
  */
  ~SDOM_Attribute(){}
  
  
  short getNodeType() const{return ATTRIBUTE;}
  const char * getNodeValue() const;
  
  /**
     * Returns the name of this attribute.
     * @since DOM Level 1
  */
  const char* getName() const;

  const char *  getNodeName() const;
  
  /**
     * The <code>DOMElement</code> node this attribute is attached to or
     * <code>null</code> if this attribute is not in use.
     *
     * @since DOM Level 2
  */
  SDOM_Node* getOwnerElement() const;
  
  
  /**
     ** NOTE - Status: Not yet implemented
     * Returns true if the attribute received its value explicitly in the
     * XML document, or if a value was assigned programatically with
     * the setValue function.  Returns false if the attribute value
     * came from the default value declared in the document's DTD.
     * @since DOM Level 1
     */
    bool  getSpecified() const{}

    /**
     * NOTE - Status: Not yet implemented
     * Returns the value of the attribute.
     *
     * The value of the attribute is returned as a string.
     * Character and general entity references are replaced with their values.
     * @since DOM Level 1
     */
    const char *       getValue() const{}

    
    /**
     * Dynamic DOM methods not supported
     * @since DOM Level 1
     */
    void          setValue(const char *value){}
  
  /**
     * NOTE - Status: Not yet implemented
     * Returns the type information associated with this attribute.
     *
     * @return the <code>DOMTypeInfo</code> associated with this attribute
     * @since DOM level 3
   */
   const SDOM_Node * getTypeInfo()const{}
};


/**
 * The <code>SDOM_Text</code> interface.
 */
class SDOM_Text: public SDOM_Node{
public:
  /**
  * Default Constructor
  */
  SDOM_Text();
  
  /**
  * Main Constructor
  * @param feature the internal node ID
  */
  //SDOM_Text(MyNodeImpl* n);
   SDOM_Text(SucTree *myTree, nodeValue n);
  /**
  * destructor
  * @param feature the internal node ID
  */
  ~SDOM_Text(){}
  
  
  short getNodeType() const{return TEXT;}
  const char * getNodeValue() const;
  
  /**
     * NOTE - Status: Not yet implemented
     *
     * @return <code>bool</code>
     * @since DOM level 3
   */
  bool isElementContentWhiteSpace();
  
  /**
     * NOTE - Status: Not yet implemented
     *
     * @return <code>bool</code>
     * @since DOM level 3
   */
  const char* wholeText();
  
  /**
     * NOTE - Status: Not yet implemented
     *
     * @return <code>bool</code>
     * @since DOM level 3
   */
  const char* split(unsigned long offset)const {}
  
  /**
     * Dynamic DOM methods not supported
     * @since DOM Level 3
  */
  const char* replaceWholeText(char* content){}
};

/**
     * NOTE - Status: Class works but does nothing more than <code>SDOM_Node</code>
     * At the moment.
*/
class SDOM_PI: public SDOM_Node{
public:
  SDOM_PI();
  SDOM_PI(SucTree *myTree, nodeValue n);//(MyNodeImpl* n);
  ~SDOM_PI(){}
  short getNodeType() const{return PROCESSING_INSTRUCTION;}
  const char * getNodeValue() const;

};


/**
     * NOTE - Status: Class works but does nothing more than <code>SDOM_Node</code>
     * At the moment.
*/
class SDOM_EntityRef: public SDOM_Node{
public:
  SDOM_EntityRef();
  SDOM_EntityRef(SucTree *myTree, nodeValue n);//(MyNodeImpl* n);
  ~SDOM_EntityRef(){}
  short getNodeType() const{return ENTITY_REFERENCE;}
  const char * getNodeValue() const;
};


/**
     * NOTE - Status: Class works but does nothing more than <code>SDOM_Node</code>
     * At the moment.
*/
class SDOM_CDataSection: public SDOM_Node{
public:
  SDOM_CDataSection();
  SDOM_CDataSection(SucTree *myTree, nodeValue n);//(MyNodeImpl* n);
  ~SDOM_CDataSection(){}
  short getNodeType() const;//{return CDATA_SECTION;}
  const char * getNodeValue() const;

  const char *  getNodeName() const;

};


/**
     * NOTE - Status: Class works but does nothing more than <code>SDOM_Node</code>
     * At the moment.
*/
class SDOM_Entity: public SDOM_Node{
public:
  SDOM_Entity();
  SDOM_Entity(SucTree *myTree, nodeValue n);//(MyNodeImpl* n);
  ~SDOM_Entity(){}
  short getNodeType() const{return ENTITY;}
  virtual   const char * getNodeValue() const;

};


/**
     * NOTE - Status: Class works but does nothing more than <code>SDOM_Node</code>
     * At the moment.
*/
class SDOM_DocType: public SDOM_Node{
public:
  SDOM_DocType();
  SDOM_DocType(SucTree *myTree, nodeValue n);//(MyNodeImpl* n);
  ~SDOM_DocType(){}
  short getNodeType() const{return DOCUMENT_TYPE;}
//  virtual const char * getNodeValue() const;

};


/**
     * NOTE - Status: Class works but does nothing more than <code>SDOM_Node</code>
     * At the moment.
*/
class SDOM_Notation: public SDOM_Node{
public:
  SDOM_Notation();
  SDOM_Notation(SucTree *myTree, nodeValue n);//(MyNodeImpl* n);
  ~SDOM_Notation(){}
  short getNodeType() const{return NOTATION;}
  virtual const char * getNodeValue() const;

};



/**
     * NOTE - Status: Not yet implemented
     *
     * @return <code>bool</code>
     * @since DOM level 3
   */
class CharacterData: public SDOM_Node{
public:
  CharacterData();
  CharacterData(SucTree *myTree, nodeValue n);
  ~CharacterData();
  unsigned long length();
  
  const char* subStringData(unsigned long offset, unsigned long count);
  
  void appendData(const char* data);

  void insertData(unsigned long offset, const char* args);
  
  void deleteData(unsigned long offset, const char* args);
};

#endif
