/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */
 
#ifndef _TREEWALKER_H
#define _TREEWALKER_H

#include "SDOM_Node.h"
#include "../Internal2/STree/myUtilsRoot.h"
class SucTree;
class STree;

/**
 * <code>TreeWalker</code> objects are used to navigate a document tree or
 * subtree using the view of the document defined by their
 * <code>whatToShow</code> flags and filter (if any). Any function which
 * performs navigation using a <code>TreeWalker</code> will automatically
 * support any view defined by a <code>TreeWalker</code>.
 * <p>Omitting nodes from the logical view of a subtree can result in a
 * structure that is substantially different from the same subtree in the
 * complete, unfiltered document. Nodes that are siblings in the
 * <code>TreeWalker</code> view may be children of different, widely
 * separated nodes in the original view. 
 * User has to define their own NodeFilter.
 * <p>See also the <a href='http://www.w3.org/TR/2000/REC-DOM-Level-2-Traversal-Range-20001113'>Document Object Model (DOM) Level 2 Traversal and Range Specification</a>.
 *
 * @since DOM Level 2
 */
class TreeWalker{
  
  
public:
  
  /**
    * Default Constructor
   */
  TreeWalker();
  
  /**
  * Main Constructor
  * @param Root ndoe to TreeWalker<code>SDOM_Node</code>
  */
  TreeWalker(SDOM_Node *node);

  /**
  * Constructor
  * @param the internal node ID
  */
//  TreeWalker(MyNodeImpl* node);
  
  TreeWalker(const TreeWalker& pnode);
   
  /**
  * destructor
  * @param feature the internal node ID
  */
  ~TreeWalker();
  
  
  /**
     * The <code>root</code> node of the <code>TreeWalker</code>, as specified
     * when it was created.
     *
     * @since DOM Level 2
  */      
  inline SDOM_Node *getRoot(){return rootNNInfo;}
   /**
     * NOTE Status: Not yet implemented
     * This attribute determines which node types are presented via the
     * <code>TreeWalker</code>. The available set of constants is defined in
     * the <code>NodeFilter</code> interface.  Nodes not accepted by
     * <code>whatToShow</code> will be skipped, but their children may still
     * be considered. Note that this skip takes precedence over the filter,
     * if any.
     *
     * @since DOM Level 2
  */
    unsigned int getWhatToShow();
   
    /**
     * NOTE Status: Not yet implemented
    */ 
    unsigned int getFilter();
    
    /**
     * NOTE Status: Not yet implemented
    */ 
    bool getExpandEntityReferences ();
    
    /**
     * Return the node at which the TreeWalker is currently positioned.
     *
     * @since DOM Level 2
     */
  SDOM_Node * getCurrentNode();
    
    /**
     * Moves to and returns the closest visible ancestor node of the current
     * node. If the search for <code>parentNode</code> attempts to step
     * upward from the <code>DTreeWalker</code>'s <code>root</code> node, or
     * if it fails to find a visible ancestor node, this method retains the
     * current position and returns <code>null</code>.
     * @return The new parent node, or <code>null</code> if the current node
     *   has no parent  in the <code>DOMTreeWalker</code>'s logical view.
     *
     * @since DOM Level 2
     */
  SDOM_Node * getParentNode();


    /**
     * Moves the <code>TreeWalker</code> to the first visible child of the
     * current node, and returns the new node. If the current node has no
     * visible children, returns <code>null</code>, and retains the current
     * node.
     * @return The new node, or <code>null</code> if the current node has no
     *   visible children  in the <code>DOMTreeWalker</code>'s logical view.
     *
     * @since DOM Level 2
    */
  SDOM_Node * getFirstChild();
    
    /**
     * Moves the <code>TreeWalker</code> to the last visible child of the
     * current node, and returns the new node. If the current node has no
     * visible children, returns <code>null</code>, and retains the current
     * node.
     * @return The new node, or <code>null</code> if the current node has no
     *   children  in the <code>TreeWalker</code>'s logical view.
     *
     * @since DOM Level 2
    */
  SDOM_Node * getLastChild();

    /**
     * Moves the <code>TreeWalker</code> to the previous sibling of the
     * current node, and returns the new node. If the current node has no
     * visible previous sibling, returns <code>null</code>, and retains the
     * current node.
     * @return The new node, or <code>null</code> if the current node has no
     *   previous sibling.  in the <code>TreeWalker</code>'s logical view.
     *
     * @since DOM Level 2
     */
  SDOM_Node * getPreviousSibling();

    /**
     * Moves the <code>TreeWalker</code> to the next sibling of the current
     * node, and returns the new node. If the current node has no visible
     * next sibling, returns <code>null</code>, and retains the current node.
     * @return The new node, or <code>null</code> if the current node has no
     *   next sibling.  in the <code>TreeWalker</code>'s logical view.
     *
     * @since DOM Level 2
     */
  SDOM_Node * getNextSibling();

    
    /**
     * Moves the <code>TreeWalker</code> to the next visible node in document
     * order relative to the current node, and returns the new node. If the
     * current node has no next node, or if the search for nextNode attempts
     * to step upward from the <code>DOMTreeWalker</code>'s <code>root</code>
     * node, returns <code>null</code>, and retains the current node.
     * @return The new node, or <code>null</code> if the current node has no
     *   next node  in the <code>DOMTreeWalker</code>'s logical view.
     *
     * @since DOM Level 2
     */
  SDOM_Node * getNextNode();
      
  
  int getNodeCount();
  
   /**
     * Moves the <code>TreeWalker</code> to the previous visible node in
     * document order relative to the current node, and returns the new
     * node. If the current node has no previous node,  or if the search for
     * <code>previousNode</code> attempts to step upward from the
     * <code>TreeWalker</code>'s <code>root</code> node,  returns
     * <code>null</code>, and retains the current node.
     * @return The new node, or <code>null</code> if the current node has no
     *   previous node in the <code>TreeWalker</code>'s logical view.
     *
     * @since DOM Level 2
     */
  SDOM_Node * getPreviousNode();

    
/**
     * The node at which the <code>TreeWalker</code> is currently positioned.
     * <code>currentNode</code> may also be explicitly set to any node,
     * whether or not it is within the subtree specified by the
     * <code>root</code> node. Further traversal occurs relative to
     * <code>currentNode</code> even if it is not part of the current view,
     * Need to complete experiements on: if no traversal
     * is possible, <code>currentNode</code> is not changed.
     * @since DOM Level 2
   */
  void setCurrentNode(SDOM_Node *currentNode);

    
 /**
 * Extension SDOM method - get current node preorder number
 */
  void setNodeNr(SDOM_Node * newVal);


protected:
  int getNodeNr();

  SDOM_Node* rootNodeImpl;
  SucTree *sucTree;
  STree* sTree;
  nodeValue rootValue;
  nodeValue tempValue;
  nodeValue currentValue;
  SDOM_Node *NNInfo;
  SDOM_Node *rootNNInfo;
  bool isText;
  int testNode;
};


#endif
